@@ -24,6 +24,8 @@ from __future__ import absolute_import, division, print_function, unicode_litera
 import math
 import os
 import re
+import gzip
+import bz2
 
 from datetime import datetime
 from calendar import timegm
@@ -33,7 +35,7 @@ from collections import OrderedDict
 
 from flent import transformers
 from flent.loggers import get_logger
-from flent.util import gzip_open, bz2_open, parse_date, format_date
+from flent.util import parse_date, format_date
 
 try:
     import ujson as json
@@ -385,17 +387,14 @@ class ResultSet(object):
 
     def dump(self, fp):
         data = self.dumps()
-        if hasattr(data, "decode"):
-            data = data.decode()
-
         return fp.write(data)
 
     def dump_file(self, filename):
         try:
             if filename.endswith(".gz"):
-                o = gzip_open
+                o = gzip.open
             elif filename.endswith(".bz2"):
-                o = bz2_open
+                o = bz2.open
             else:
                 o = open
             with o(filename, "wt") as fp:
@@ -434,9 +433,9 @@ class ResultSet(object):
         self._dump_file = os.path.join(dirname, self._gen_filename())
         try:
             if self._dump_file.endswith(".gz"):
-                o = gzip_open
+                o = gzip.open
             elif self._dump_file.endswith(".bz2"):
-                o = bz2_open
+                o = bz2.open
             else:
                 o = open
             with o(self._dump_file, "wt") as fp:
@@ -508,7 +507,7 @@ class ResultSet(object):
                 # relying on their presence in future code.
                 logger.debug("No raw values found; synthesising from parsed data")
 
-                t0 = parse_date(obj['metadata'].get(
+                t0, offset = parse_date(obj['metadata'].get(
                     'T0', obj['metadata'].get('TIME')))
                 x0 = timegm(t0.timetuple()) + t0.microsecond / 1000000.0
                 for name in obj['results'].keys():
@@ -627,9 +626,9 @@ class ResultSet(object):
         logger.debug("Loading data file %s", filename)
         try:
             if filename.endswith(".gz"):
-                o = gzip_open
+                o = gzip.open
             elif filename.endswith(".bz2") or filename.endswith(".flnt"):
-                o = bz2_open
+                o = bz2.open
             else:
                 o = open
             fp = o(filename, 'rt')
